//
// Created by draze on 3/5/17.
//

#ifndef FS_METADATA_OPS_H
#define FS_METADATA_OPS_H

#include "../main.h"
#include "../classes/metadata.h"

using namespace std;

#include <boost/archive/binary_iarchive.hpp>
#include <boost/archive/binary_oarchive.hpp>

// mapping of enum to string to get the file names for metadata
enum class Md_fields { atime, mtime, ctime, uid, gid, mode, inode_no, link_count, size, blocks };

const std::map<Md_fields, std::string> md_field_map = {
        {Md_fields::atime, "/atime"},
        {Md_fields::mtime, "/mtime"},
        {Md_fields::ctime, "/ctime"},
        {Md_fields::uid, "/uid"},
        {Md_fields::gid, "/gid"},
        {Md_fields::mode, "/mode"},
        {Md_fields::inode_no, "/inode_no"},
        {Md_fields::link_count, "/link_count"},
        {Md_fields::size, "/size"},
        {Md_fields::blocks, "/blocks"}
};

bool write_all_metadata(const Metadata& md, const unsigned long hash);

// TODO error handling.
template<typename T>
bool write_metadata_field(const T& field, const unsigned long hash, const string& leaf_name) {
    auto i_path = bfs::path(ADAFS_DATA->inode_path);
    i_path /= to_string(hash);
    bfs::create_directories(i_path);

    i_path /= leaf_name;
    // for some reason auto ofs = bfs::ofstream(file); does not work. That is why I use the old style.
    // std::basic_ofstream does not encounter this problem
    bfs::ofstream ofs{i_path};
    // write to disk in binary form
    boost::archive::binary_oarchive ba(ofs);
    ba << field;

    return true;
}

bool read_all_metadata(Metadata& md, const unsigned long hash);

template<typename T>
unique_ptr<T> read_metadata_field(const unsigned long hash, const string& leaf_name) {
    auto path = bfs::path(ADAFS_DATA->inode_path);
    path /= to_string(hash);
    path /= leaf_name;
    if (!bfs::exists(path)) return nullptr;

    bfs::ifstream ifs{path};
    //fast error checking
    //ifs.good()
    boost::archive::binary_iarchive ba(ifs);
    auto field = make_unique<T>();
    ba >> *field;
    return field;
}

int get_metadata(Metadata& md, const std::string& path);

int get_metadata(Metadata& md, const boost::filesystem::path& path);

int remove_metadata(const unsigned long hash);

#endif //FS_METADATA_OPS_H
